;                     MEGASQUIRT-AVR
; assembly.S
; assembly language bootloader commands for the Genboard ATMEGA16
; 
#include "defines.h"
.global page_erase
page_erase:                 ; 
        RCALL   WAIT_SPMEN  ;Wait for SPMEN flag cleared
        MOVW    R30,R24     ;move address to z pointer (R31=ZH R30=ZL)
        LDI     R18,0x03    ; 3 = page erase, data sheet pg 278
        OUT 	0x37, R18   ; load spmcsr with page erase command
        SPM                 ;erase the page
        RET

.global page_write
page_write:
        RCALL   WAIT_SPMEN  ;Wait for SPMEN flag cleared
        MOVW    R30,R24     ;move address to z pointer (R31=ZH R30=ZL)
        LDI     R18,0x05    ; 5 = page write, data sheet pg 277
        OUT 	0x37, R18   ; load spmcr with page write command
        SPM                 ;write the page
        RET

; why isn't the Z-pointer setup here?
.global reenable_rww 
reenable_rww:
        RCALL   WAIT_SPMEN  ;Wait for SPMEN flag cleared
        LDI     R18,0x11    ; 0x11 = re-enable rww, data sheet pg 277
        OUT 	0x37, R18   ; load spmcr with re-enable command
        SPM                 ;do the re-enable
        RET

.global fill_temp_buffer
fill_temp_buffer:           ; (data,address);
        RCALL   WAIT_SPMEN  ;Wait for SPMEN flag cleared       
        MOVW    R30,R22     ;move address to z pointer (R31=ZH R30=ZL)
        MOVW    R0,R24      ;move data to reg 0 and 1 
        LDI     R18,0x01    ; (uint8_t)SPMEN_BIT
        OUT 	0x37, R18   ;load spmcsr with buffer fill command
        SPM                 ;Store the data
        EOR     R1, R1      ;restore zero
        RET
          
 
.global read_prog_mem_word
read_prog_mem_word:
        RCALL   WAIT_SPMEN
        MOVW    R30,R24     ;move address to z pointer
        LPM    R24,Z+      ;read LSB  use LPM's for smaller mega parts
        LPM    R25,Z+      ;read MSB
        RET
        
WAIT_SPMEN:
        IN     	R18,0x37        ;get SPMCR register
        SBRC    R18,0           ;test the SPMEN bit
        RJMP    WAIT_SPMEN      ;Wait for SPMEN bit cleared

        RET
        
; MIK:
; This is a cheatcode!
; place the jmp instruction at specified address thereby
; eliminating the need to manually edit the hex-file
 .section .bootjmp,"ax",@progbits
	jmp 0x3800

	
